@@ -243,10 +243,10 @@ ImgBmapCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
  */
 
 static int
-ImgBmapConfigureMaster(masterPtr, argc, objv, flags)
+ImgBmapConfigureMaster(masterPtr, objc, objv, flags)
     BitmapMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall bitmap image to (reconfigure). */
-    int argc;			/* Number of entries in argv. */
+    int objc;			/* Number of entries in objv. */
     Tcl_Obj *CONST objv[];	/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
@@ -254,14 +254,14 @@ ImgBmapConfigureMaster(masterPtr, argc, objv, flags)
     BitmapInstance *instancePtr;
     int maskWidth, maskHeight, dummy1, dummy2;
 
-    char **argv = (char **) ckalloc((argc+1) * sizeof(char *));
-    for (dummy1 = 0; dummy1 < argc; dummy1++) {
-	argv[dummy1]=Tcl_GetStringFromObj(objv[dummy1], (int *) NULL);
+    char **argv = (char **) ckalloc((objc+1) * sizeof(char *));
+    for (dummy1 = 0; dummy1 < objc; dummy1++) {
+	argv[dummy1]=Tcl_GetString(objv[dummy1]);
     }
-    argv[argc] = NULL;
+    argv[objc] = NULL;
 
     if (Tk_ConfigureWidget(masterPtr->interp, Tk_MainWindow(masterPtr->interp),
-	    configSpecs, argc, argv, (char *) masterPtr, flags)
+	    configSpecs, objc, argv, (char *) masterPtr, flags)
 	    != TCL_OK) {
 	ckfree((char *) argv);
 	return TCL_ERROR;
@@ -755,43 +755,43 @@ NextBitmapWord(parseInfoPtr)
  */
 
 static int
-ImgBmapCmd(clientData, interp, argc, argv)
+ImgBmapCmd(clientData, interp, objc, objv)
     ClientData clientData;	/* Information about the image master. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    int argc;			/* Number of arguments. */
-    Tcl_Obj *CONST argv[];	/* Argument objects. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument objects. */
 {
     static char *bmapOptions[] = {"cget", "configure", (char *) NULL};
     BitmapMaster *masterPtr = (BitmapMaster *) clientData;
     int code, index;
 
-    if (argc < 2) {
-	Tcl_WrongNumArgs(interp, 1, argv, "option ?arg arg ...?");
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option ?arg arg ...?");
 	return TCL_ERROR;
     }
-    if (Tcl_GetIndexFromObj(interp, argv[1], bmapOptions, "option", 0,
+    if (Tcl_GetIndexFromObj(interp, objv[1], bmapOptions, "option", 0,
 	    &index) != TCL_OK) {
 	return TCL_ERROR;
     }
     switch (index) {
       case 0: {
-	if (argc != 3) {
-	    Tcl_WrongNumArgs(interp, 2, argv, "option");
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "option");
 	    return TCL_ERROR;
 	}
 	return Tk_ConfigureValue(interp, Tk_MainWindow(interp), configSpecs,
-		(char *) masterPtr, Tcl_GetStringFromObj(argv[2], NULL), 0);
+		(char *) masterPtr, Tcl_GetString(objv[2]), 0);
       }
       case 1: {
-	if (argc == 2) {
+	if (objc == 2) {
 	    code = Tk_ConfigureInfo(interp, Tk_MainWindow(interp),
 		    configSpecs, (char *) masterPtr, (char *) NULL, 0);
-	} else if (argc == 3) {
+	} else if (objc == 3) {
 	    code = Tk_ConfigureInfo(interp, Tk_MainWindow(interp),
 		    configSpecs, (char *) masterPtr,
-		    Tcl_GetStringFromObj(argv[2], NULL), 0);
+		    Tcl_GetString(objv[2]), 0);
 	} else {
-	    code = ImgBmapConfigureMaster(masterPtr, argc-2, argv+2,
+	    code = ImgBmapConfigureMaster(masterPtr, objc-2, objv+2,
 		    TK_CONFIG_ARGV_ONLY);
 	}
 	return code;
